using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace GLCDView
{
	public partial class GLCDFontEditForm : Form
	{
		GLCDFont glcdFont = null;

		public GLCDFontEditForm()
		{
			InitializeComponent();
		}

		public GLCDFont GLCDFont
		{
			get { return this.glcdFont; }
			set { this.glcdFont = value; }
		}

		private void GLCDFontEditForm_Load(object sender, EventArgs e)
		{
			if (null != this.glcdFont)
			{
				if (null != this.glcdFont.Font)
					this.textBoxChars.Font = this.glcdFont.Font;
				this.textBoxChars.Text = this.glcdFont.Chars;
				this.textBoxPaddingLeft.Text = this.glcdFont.PaddingLeft.ToString();
				this.textBoxPaddingRight.Text = this.glcdFont.PaddingRight.ToString();
				this.textBoxPaddingTop.Text = this.glcdFont.PaddingTop.ToString();
				this.textBoxPaddingBottom.Text = this.glcdFont.PaddingBottom.ToString();
				this.textBoxTopBottomReference.Text = this.glcdFont.Reference;
				this.checkBoxFixedSize.Checked = this.glcdFont.FixedSize;
			}
			else
			{
				this.textBoxChars.Text = GLCDFont.DefaultChars;
				this.textBoxTopBottomReference.Text = GLCDFont.DefaultReference;
			}


			UpdateFontInfo(this.textBoxChars.Font);
			this.buttonOK.Enabled = false;
		}

		private void UpdateFontInfo(Font font)
		{
			string fontInfo = string.Format("{0}, {1}\n{2}", font.Name, font.Size, font.Style);

			this.labelFontInfo.Text = fontInfo;
			this.textBoxChars.Font = font;

			int s = (int)font.Size;
			this.textBoxName.Text = font.Name.Replace(' ', '_')+"__"+s.ToString();
		}

		private void buttonSelectFont_Click(object sender, EventArgs e)
		{
			FontDialog dlg = new FontDialog();
			dlg.Font = this.textBoxChars.Font;
			dlg.AllowVectorFonts = true;
			dlg.FixedPitchOnly = false;
			dlg.ScriptsOnly = false;

			if (dlg.ShowDialog(this) != DialogResult.OK)
				return;
			
			UpdateFontInfo(dlg.Font);
		}

		private void buttonGenerate_Click(object sender, EventArgs e)
		{
			Cursor restore = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;

			this.glcdFont = new GLCDFont();
			this.glcdFont.Font = this.textBoxChars.Font;

			this.richTextBoxCode.Text = string.Empty;

			this.glcdFont.Name = string.IsNullOrEmpty(this.textBoxName.Text) ? this.glcdFont.Font.FontFamily.Name.Replace(' ', '_') : this.textBoxName.Text;
			this.glcdFont.PaddingLeft = Convert.ToInt32(this.textBoxPaddingLeft.Text);
			this.glcdFont.PaddingRight = Convert.ToInt32(this.textBoxPaddingRight.Text);
			this.glcdFont.PaddingTop = Convert.ToInt32(this.textBoxPaddingTop.Text);
			this.glcdFont.PaddingBottom = Convert.ToInt32(this.textBoxPaddingBottom.Text);
			this.glcdFont.Reference = this.textBoxTopBottomReference.Text;
			this.glcdFont.FixedSize = this.checkBoxFixedSize.Checked;
			bool savePix = -1 != this.glcdFont.Reference.IndexOf("DEBUG");

			// string txt = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890.,;:-_+*#?!^/()&%$@|<>";
			string txt = this.textBoxChars.Text.Replace("\r\n", "");
			this.glcdFont.Chars = txt;
			StringBuilder trace;

			Bitmap bmp = new Bitmap(300, 300);
			Graphics g = Graphics.FromImage(bmp);

			int top = -1, bottom = -1;
			FindTopAndBottom(g, bmp, this.glcdFont.Font, ref top, ref bottom);

			for (int i = 0; i < txt.Length; i++)
			{
				CharData charData = new CharData();
				string b = txt.Substring(i, 1);
				SizeF s = g.MeasureString(b, this.glcdFont.Font);
				int h = (int)s.Height;
				h += s.Height * 10 - h * 10 >= 1 ? 1 : 0;
				int w = (int)s.Width;
				w += s.Width * 10 - w * 10 >= 1 ? 1 : 0;

				#region prepare the bitmap
				g.FillRectangle(Brushes.Red, 0, 0, 300, 300);
				g.FillRectangle(Brushes.White, 0, 0, w, h);
				g.DrawString(b, this.glcdFont.Font, Brushes.Black, 0, 0);
				#endregion

				byte[] bb = Encoding.Default.GetBytes(b);
				string path = string.Format("pic0x{0}.bmp", bb[0].ToString("x"));

				Color c;
				int left = -1;
				int right = -1;

				#region get left and right padding value
				if (b != " ")
				{
					for (int x = 0; x < w; x++)
					{
						for (int y = 0; y < h; y++)
						{
							c = bmp.GetPixel(x, y);
							if (c.R != 0xff && c.G != 0xff && c.B != 0xff)
							{
								if (left == -1)
								{
									left = x;
									break;
								}
							}
						}
					}
					for (int x = w - 1; x > 0; x--)
					{
						for (int y = 0; y < h; y++)
						{
							c = bmp.GetPixel(x, y);
							if (c.R != 0xff && c.G != 0xff && c.B != 0xff)
							{
								if (right == -1)
								{
									right = x;
									break;
								}
							}
						}
					}

					left -= this.glcdFont.PaddingLeft;
					if (left < 0)
						left = 0;
					right += this.glcdFont.PaddingRight;
					if (right > w)
						right = w;
				}
				else
				{
					// for "blank"
					left = 2;
					right = w;
				}
				#endregion

				charData.Char = bb[0];
				charData.Width = right - left + 1;
				charData.Height = bottom - top + 1;

				int byteCount = ((charData.Width - 1) / 8) + 1;
				
				charData.Comment = string.Format("'{0}' Width:{1} Height:{2}, Bytes:{3}", b, charData.Width, charData.Height, byteCount);

				charData.DataRows = new CharDataRow[charData.Height];

				int idx, yIdx = 0;
				for (int y = top; y < bottom + 1; y++)
				{
					idx = 0;
					charData.DataRows[yIdx].Data = new byte[byteCount];
					trace = new StringBuilder();

					for (int x = left; x <= right; x++)
					{
						if (idx % 8 == 0)
							trace.Append(' ');

						c = bmp.GetPixel(x, y);
						if (c.R != 0xff && c.G != 0xff && c.B != 0xff)
						{
							// pixel set!
							//System.Diagnostics.Debug.Write("#");
							trace.Append('#');

							int dI = idx / 8;
							byte v = charData.DataRows[yIdx].Data[dI];
							int r = idx % 8;
							v |= (byte)(0x80 >> r);
							charData.DataRows[yIdx].Data[dI] = v;
							
							bmp.SetPixel(x, y, Color.LightGreen);
						}
						else
						{
							//System.Diagnostics.Debug.Write("");
							trace.Append('');
							bmp.SetPixel(x, y, Color.Green);
						}
						idx++; // bit index
					}

					charData.DataRows[yIdx].Comment = trace.ToString();
					yIdx++;
				}

				#region save the graphic
				if(savePix)
					bmp.Save(path);
				#endregion

				this.glcdFont.CharData.Add(charData);
			}
			g.Dispose();
			bmp.Dispose();

			if (this.glcdFont.FixedSize)
			{
				#region calculate the biggest width
				int maxWidth = 0;
				string s1 = string.Empty;
				byte[] b1 = new byte[1];

				foreach (CharData cd in this.glcdFont.CharData)
				{
					if (cd.Width > maxWidth)
					{
						maxWidth = cd.Width;
						b1[0] = cd.Char;
						s1 += Encoding.Default.GetString(b1);
					}
				}
				#endregion
				#region recalculate propotional font
				for (int oldIdx = 0; oldIdx < this.glcdFont.CharData.Count; oldIdx++)
				{
					CharData cd = this.glcdFont.CharData[oldIdx];
					if (cd.Width >= maxWidth)
						continue;

					int offsetLeft = (maxWidth - cd.Width) / 2;
					int addRight = (maxWidth - cd.Width) / 2 + (maxWidth - cd.Width) % 2;
					int byteCount = ((maxWidth - 1) / 8) + 1;
					b1[0] = cd.Char;

					cd.Comment = string.Format("'{0}' Width:{1} Height:{2}, Bytes:{3}", Encoding.Default.GetString(b1), maxWidth, cd.Height, byteCount);

					CharDataRow[] newRows = new CharDataRow[cd.Height];

					int idx, yIdx = 0;
					foreach (CharDataRow dr in cd.DataRows)
					{
						newRows[yIdx].Data = new byte[byteCount];

						idx = offsetLeft;
						trace = new StringBuilder();

						for (int x = 0; x < idx; x++)
						{
							if (x % 8 == 0)
								trace.Append(' ');
							trace.Append('');
						}

						for (int x = 0; x < cd.Width; x++)
						{
							if (idx % 8 == 0)
								trace.Append(' ');

							if ((0x80 >> (x % 8)) == (dr.Data[x / 8] & (0x80 >> (x % 8))))
							{
								// pixel set!
								//System.Diagnostics.Debug.Write("#");
								trace.Append('#');

								int dI = idx / 8;
								byte v = newRows[yIdx].Data[dI];
								int r = idx % 8;
								v |= (byte)(0x80 >> r);
								newRows[yIdx].Data[dI] = v;
							}
							else
							{
								//System.Diagnostics.Debug.Write("");
								trace.Append('');
							}
							idx++; // bit index
						}


						for (int x = idx; x < maxWidth; x++)
						{
							if (x % 8 == 0)
								trace.Append(' ');
							trace.Append('');
						}


						newRows[yIdx].Comment = trace.ToString();
						yIdx++;
					}

					cd.Width = maxWidth;
					cd.DataRows = newRows;
				}

				#endregion
			}

			this.glcdFont.glyphHeight = this.glcdFont.CharData[0].Height;
			this.glcdFont.glyphWidth = 0;

			this.richTextBoxCode.Text = this.glcdFont.ExportSourceFile(null);

			this.buttonOK.Enabled = true;
			Cursor.Current = restore;
		}

		private void FindTopAndBottom(Graphics g, Bitmap bmp, Font f, ref int top, ref int bottom)
		{
			g.FillRectangle(Brushes.Red, 0, 0, 300, 300);

			SizeF s = g.MeasureString(this.glcdFont.Reference, f);
			int h = (int)s.Height;
			h += s.Height * 10 - h * 10 >= 1 ? 1 : 0;
			int w = (int)s.Width;
			w += s.Width * 10 - w * 10 >= 1 ? 1 : 0;

			g.FillRectangle(Brushes.White, 0, 0, w, h);
			g.DrawString(this.glcdFont.Reference, f, Brushes.Black, 0, 0);

			Color c;
			for (int y = 0; y < h; y++)
			{
				for (int x = 0; x < w; x++)
				{
					c = bmp.GetPixel(x, y);
					if (c.R != 0xff && c.G != 0xff && c.B != 0xff)
					{
						if (top == -1)
						{
							top = y;
							break;
						}
					}
				}
			}
			for (int y = h-1; y > 0; y--)
			{
				for (int x = 0; x < w; x++)
				{
					c = bmp.GetPixel(x, y);
					if (c.R != 0xff && c.G != 0xff && c.B != 0xff)
					{
						if (bottom == -1)
						{
							bottom = y;
							break;
						}
					}
				}
			}

			top -= this.glcdFont.PaddingTop;
			if (top < 0)
				top = 0;
			bottom += this.glcdFont.PaddingBottom;
			if (bottom > h)
				bottom = h;
		}

		private void buttonOK_Click(object sender, EventArgs e)
		{
			this.DialogResult = DialogResult.OK;
			Close();
		}

		private void buttonCancel_Click(object sender, EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
			Close();
		}
	}
}