using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using GLCDView;

namespace GLCD
{
	public partial class MainForm : Form
	{
		private GLCDFont currentFont = null;

		public MainForm()
		{
			InitializeComponent();
		}

		protected override void OnClosing(CancelEventArgs e)
		{
			foreach (GLCDFont f in this.comboBoxFonts.Items)
			{
				string fname, data = f.ToData();
				if ( ! string.IsNullOrEmpty(data))
				{
					fname = string.Format("{0}.glcdf", f.Name);
					try
					{
						using (StreamWriter sw = new StreamWriter(fname, false, Encoding.Default))
						{
							sw.Write(data);
						}
					}
					catch (Exception)
					{
					}
				}
			}
			base.OnClosing(e);
		}

		protected override void OnLoad(EventArgs e)
		{
			this.textBoxXDim.Text = this.glcdView1.XDimension.ToString();
			this.textBoxYDim.Text = this.glcdView1.YDimension.ToString();

			string[] fonts = Directory.GetFiles("./", "*.glcdf");

			foreach (string fname in fonts)
			{
				string data;
				using (StreamReader r = new StreamReader(fname, Encoding.Default))
				{
					data = r.ReadToEnd();
				}

				GLCDFont f = GLCDFont.FromData(data);
				if (null != f)
					this.comboBoxFonts.Items.Add(f);
			}


			base.OnLoad(e);
		}

		private void buttonPrint_Click(object sender, EventArgs e)
		{
			if (null == this.currentFont)
				return;
			if(string.IsNullOrEmpty(this.textBoxXOffset.Text)
				|| string.IsNullOrEmpty(this.textBoxYOffset.Text))
				return;

			Print(Convert.ToInt32(this.textBoxXOffset.Text),
				Convert.ToInt32(this.textBoxYOffset.Text),
				this.textBoxText.Text, this.checkBoxInvers.Checked);
			this.glcdView1.Invalidate();
		}

		private void buttonLine_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(this.textBoxLineX1.Text)
				|| string.IsNullOrEmpty(this.textBoxLineY1.Text)
				|| string.IsNullOrEmpty(this.textBoxLineX2.Text)
				|| string.IsNullOrEmpty(this.textBoxLineY2.Text))
				return;

			int x1 = Convert.ToInt32(this.textBoxLineX1.Text);
			int y1 = Convert.ToInt32(this.textBoxLineY1.Text);
			int x2 = Convert.ToInt32(this.textBoxLineX2.Text);
			int y2 = Convert.ToInt32(this.textBoxLineY2.Text);
			bool set = !this.checkBoxInvers.Checked;
			this.glcdView1.DrawLine(x1,y1,x2,y2,set);

			string trace = string.Format("glcd_line({0}, {1}, {2}, {3}, {4});\r\n", x1, y1, x2, y2, Convert.ToInt32(set));
			this.richTextBoxTrace.Text += trace;
		}

		private void buttonRectangle_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(this.textBoxLineX1.Text)
				|| string.IsNullOrEmpty(this.textBoxLineY1.Text)
				|| string.IsNullOrEmpty(this.textBoxLineX2.Text)
				|| string.IsNullOrEmpty(this.textBoxLineY2.Text))
				return;

			int x1 = Convert.ToInt32(this.textBoxLineX1.Text);
			int y1 = Convert.ToInt32(this.textBoxLineY1.Text);
			int x2 = Convert.ToInt32(this.textBoxLineX2.Text);
			int y2 = Convert.ToInt32(this.textBoxLineY2.Text);
			bool set = !this.checkBoxInvers.Checked;
			this.glcdView1.DrawRectangle(x1, y1, x2, y2, set);

			string trace = string.Format("glcd_box({0}, {1}, {2}, {3}, {4});\r\n", x1, y1, x2, y2, Convert.ToInt32(set));
			this.richTextBoxTrace.Text += trace;
		}

		private void buttonFilled_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(this.textBoxLineX1.Text)
				|| string.IsNullOrEmpty(this.textBoxLineY1.Text)
				|| string.IsNullOrEmpty(this.textBoxLineX2.Text)
				|| string.IsNullOrEmpty(this.textBoxLineY2.Text))
				return;

			int x1 = Convert.ToInt32(this.textBoxLineX1.Text);
			int y1 = Convert.ToInt32(this.textBoxLineY1.Text);
			int x2 = Convert.ToInt32(this.textBoxLineX2.Text);
			int y2 = Convert.ToInt32(this.textBoxLineY2.Text);
			bool set = !this.checkBoxInvers.Checked;

			this.glcdView1.FillRectangle(x1, y1, x2, y2, set);

			string trace = string.Format("glcd_box_filled({0}, {1}, {2}, {3}, {4});\r\n", x1, y1, x2, y2, Convert.ToInt32(set));
			this.richTextBoxTrace.Text += trace;
		}

		private void buttonClearGfx_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(this.textBoxXDim.Text)
				|| string.IsNullOrEmpty(this.textBoxYDim.Text))
				return;

			this.glcdView1.XDimension = Convert.ToInt32(this.textBoxXDim.Text);
			this.glcdView1.YDimension = Convert.ToInt32(this.textBoxYDim.Text);

			
			
			
			this.glcdView1.ClearGfx();
		}

		private void checkBoxGrid_CheckedChanged(object sender, EventArgs e)
		{
			this.glcdView1.ShowGrid = this.checkBoxGrid.Checked;
		}

		int lastX=0, rememberX=0, lastY=0, rememberY=0;
		private void glcdView1_PixelChanged(object sender, GLCDView.PixelChangedArgs e)
		{
			this.lastX = e.XPixel;
			this.lastY = e.YPixel;
			this.labelPixelInfo.Text = string.Format("{0},{1}", e.XPixel, e.YPixel);
		}

		private void glcdView1_MouseDown(object sender, MouseEventArgs e)
		{
			this.rememberX = this.lastX;
			this.rememberY = this.lastY;
		}

		private void textBoxXOffset_DoubleClick(object sender, EventArgs e)
		{
			this.textBoxXOffset.Text = this.rememberX.ToString();
			this.textBoxYOffset.Text = this.rememberY.ToString();
		}

		private void textBoxLineX1_DoubleClick(object sender, EventArgs e)
		{
			this.textBoxLineX1.Text = this.rememberX.ToString();
			this.textBoxLineY1.Text = this.rememberY.ToString();
		}

		private void textBoxLineX2_DoubleClick(object sender, EventArgs e)
		{
			this.textBoxLineX2.Text = this.rememberX.ToString();
			this.textBoxLineY2.Text = this.rememberY.ToString();
		}

		private void buttonFont_Click(object sender, EventArgs e)
		{
			GLCDFontEditForm dlg = new GLCDFontEditForm();
			dlg.GLCDFont = this.currentFont;

			if (dlg.ShowDialog(this) == DialogResult.OK)
			{
				this.comboBoxFonts.Items.Add(dlg.GLCDFont);
			}
		}

		public void Print(int x, int y, string text, bool invers)
		{
			string trace = string.Format("glcd_print2_P({0}, {1}, \"{2}\", &{3}, {4});\r\n", x, y, text, currentFont.Name, Convert.ToInt32(invers).ToString());
			this.richTextBoxTrace.Text += trace;
			
			int offset, width, j, allwidth = 0;
			byte i, ertefah, map;

			byte[] byteText = Encoding.Default.GetBytes(text);
			for (int idx = 0; idx < byteText.Length; idx++)
			{
				map = byteText[idx];
				map = currentFont.mapping_table[map];
				width = currentFont.glyphWidth;
				if (width == 0)
					width = currentFont.width_table[map];

				offset = currentFont.offset_table[map];
				ertefah = (byte)currentFont.glyphHeight;

				for (j = 0; j < ertefah * (((width - 1) / 8) + 1); j += (((width - 1) / 8) + 1))
				{   // ertefah
					for (i = 0; i < width; i++)
					{   //  width
						if (0 != (currentFont.char_table[offset + j + (i / 8)] & (1 << (7 - (i % 8)))))
							this.glcdView1.SetPixel(x + i + allwidth, y + j / (((width - 1) / 8) + 1), !invers);
						else
							this.glcdView1.SetPixel(x + i + allwidth, y + j / (((width - 1) / 8) + 1), invers);
					}//End i
				}// End j
				allwidth += width;
			}// End K
		}

		private void comboBoxFonts_SelectedIndexChanged(object sender, EventArgs e)
		{
			this.currentFont = this.comboBoxFonts.SelectedItem as GLCDFont;
			if (null != this.currentFont)
				this.labelSize.Text = string.Format("Size = {0} Bytes", this.currentFont.Size);
			else
				this.labelSize.Text = string.Empty;
		}

		private void buttonFontExport_Click(object sender, EventArgs e)
		{
			if (null == this.currentFont)
				return;

			SaveFileDialog dlg = new SaveFileDialog();
			dlg.FileName = this.currentFont.Name + ".c";
			if (dlg.ShowDialog(this) == DialogResult.OK)
			{
				this.currentFont.ExportSourceFile(dlg.FileName);
			}
		}

		private void buttonFontDelete_Click(object sender, EventArgs e)
		{
			if (null != this.currentFont)
			{
				string fname = string.Format("{0}.glcdf", this.currentFont.Name);
				if (File.Exists(fname))
					File.Delete(fname);
				this.comboBoxFonts.Items.Remove(this.currentFont);
			}
		}
	}
}